% This file generate a cell state space larger than the range of interest.
clear all
close all
clc
% global m k c kStar cStar g w Fs Fc f
global delta
addpath('functions');

%% System parameters
zeta = 0.015;
zeta_star = 0.015;
rho_star = 2;

rho = 1.0;
delta = 10;

%% Options
options1 = odeset('RelTol',1e-6,'AbsTol',1e-6,'Events',@event_f1); % option for state1
options2 = odeset('RelTol',1e-6,'AbsTol',1e-6,'Events',@event_f2); % option for state2

Nc = 1000; % Number of max. integration cycles
CC = 1; % Convergence criteria (%)
threshold = 20; % A threshold value for distinguishing the nonlinear & linear responses 
%% Set up range of interest
x1_u = 40;
x1_l = -40.01; 
x2_u = 40;
x2_l = -40.01;

Nx1 = 20; Nx2 = 20; % Grid number in u & u' direction

% Create initial guess matrix
ICList_x1 = linspace(x1_l,x1_u,Nx1);
ICList_x2 = linspace(x2_l,x2_u,Nx2); 
% Amplitude matrix
Amp_mat = zeros(Nx2,Nx1);

%% Determine the basin of attraction
tic
for j = 1:Nx2
    j
     for i = 1:Nx1
        IC = [ICList_x1(i),ICList_x2(j)];
        [Amp,xcList,tcList] = SDOF_Attractor_func(IC,zeta,zeta_star,rho,rho_star,Nc,CC,options1,options2);
       
        Amp_mat(j,i) = Amp;
     end
end
toc

%% Post-processing data and visualization
% Seperate each amplitude group and save them seperately
LinInd = find(Amp_mat < threshold);
[Ind_x2_lin,Ind_x1_lin] = ind2sub(size(Amp_mat),LinInd);
BOA_x1_lin = ICList_x1(Ind_x1_lin); BOA_x2_lin = ICList_x2(Ind_x2_lin);

LinInd = find(Amp_mat > threshold);
[Ind_x2_bilin,Ind_x1_bilin] = ind2sub(size(Amp_mat),LinInd);
BOA_x1_bilin = ICList_x1(Ind_x1_bilin); BOA_x2_bilin = ICList_x2(Ind_x2_bilin);


%% Plot
hold on
plot(BOA_x1_lin,BOA_x2_lin,'k.') % linear response
plot(BOA_x1_bilin,BOA_x2_bilin,'rs','markersize',3.5) % nonlinear response
box on
axis([x1_l,x1_u,x2_l,x2_u])

xlabel('Displacement (u)')
ylabel('Velocity (u'')')

legend(['Amplitude = ',num2str(min(min(Amp_mat)))], ['Amplitude = ',num2str(max(max(Amp_mat)))]) % MATLAB 2017b
title(['\rho = ',num2str(rho), ' & \delta = ', num2str(delta)])

