% function [FS,tcList,xcList] = SDOF_CM_function(IC,m,k,c,kStar,cStar,w,Fs,Fc,f) % Use this for debugging purpose (plot entire time history)
function [Amp,uList,tauList] = SDOF_Attractor_func(IC,zeta,zeta_star,rho,rho_star,Nc,CC,options1,options2)
global delta

% Determine the first state
if IC(1) < delta
    state = 1;
elseif IC(1) > delta
    state = 2;
else
    disp('Cannot determine the first state')
end

%% Establish empty vector & parameters
tauList=[]; uList=[]; vList=[]; taueList = []; AmpList = [0]; RelErrList = []; flag = 0;
tauInitial=0; 
T = 2*pi/rho; % Excitation period
tauTerminate = Nc*T;
tauEvalPt = [T:T:tauTerminate-T]; % Evaluate the dis. & vel. over each excitation period

%% Time  integrate to steady state (ss)
while tauInitial < tauTerminate
    if state == 1 % closed state
        del = tauEvalPt < tauInitial; % Find time points which should be deleted
        tauEvalPt(del) = [];
        
        du1 = @(tau1,u1) [u1(2);rho^2*sin(rho*tau1) - 2*zeta*u1(2) - u1(1)];
        [tau1,u1,taue,ue,ie] = ode45(du1,[tauInitial,tauEvalPt,tauInitial+tauTerminate],[IC],options1);
        
        tauList = [tauList;tau1(2:end-1)];
        uList = [uList;u1(2:end-1,1)];
        vList = [vList;u1(2:end-1,2)];
        taueList = [taueList;taue];
        
        tauInitial = tau1(end); % Initial time for next stage
        IC = u1(end,:);; % Update initial conditions
        
%         if tau1(end) == tauInitial + tauTerminate % No transition occurs -> Stop integration
%             IC = u1(:,1); % Update initial conditions
%             break
%         end
        
        % Determine next state
        if ie == 1
            state = 2;
        else
%             disp('No transition in last state')
        end
    %--------------------------------------------------------------------------------    
    elseif state == 2 % open state
        del = tauEvalPt < tauInitial; % Find time points which should be deleted
        tauEvalPt(del) = [];
        
        du2 = @(tau2,u2) [u2(2);rho^2*sin(rho*tau2) - (2*zeta+2*zeta_star*rho_star)*u2(2) - (1+rho_star^2)*u2(1) + delta*rho_star^2];
        [tau2,u2,taue,ue,ie] = ode45(du2,[tauInitial,tauEvalPt,tauInitial+tauTerminate],[IC],options2);
        
        tauList = [tauList;tau2(2:end-1)];
        uList = [uList;u2(2:end-1,1)];
        vList = [vList;u2(2:end-1,2)];
        taueList = [taueList;taue];
        
        tauInitial = tau2(end); % Initial time for next stage
        IC = u2(end,:); % Update initial conditions
        
        
        
%         if tau2(end) == tauInitial + tauTerminate % No transition occurs -> Stop integration
%             IC = u2(:,1); % Update initial conditions
%             break
%         end
        
        % Update the state
        if ie == 1
            state = 1;
        else
%             disp('No transition in last state')
        end
        
    %--------------------------------------------------------------------------------    
    else
        disp('Unexpected state determination')
    end
    %---------------------------------------------------------------------------------
    
    
    % Check convergence criterion
    % Check three points
    if length(uList) > 5
        
        uDiff1 = (uList(end)-uList(end-1))/uList(end-1)*100;
        vDiff1 = (vList(end)-vList(end-1))/vList(end-1)*100;
        uDiff2 = (uList(end-1)-uList(end-2))/uList(end-2)*100;
        vDiff2 = (vList(end-1)-vList(end-2))/vList(end-2)*100;
        uDiff3 = (uList(end-2)-uList(end-3))/uList(end-3)*100;
        vDiff3 = (vList(end-2)-vList(end-3))/vList(end-3)*100;
        
        if abs(uDiff1) < CC && abs(vDiff1) < CC && abs(uDiff2) < CC && abs(vDiff2) < CC && abs(uDiff3) < CC && abs(vDiff3) < CC % Meet convergence criteria
            flag = 1;
        end
    end
    if flag == 1
        break
    end
end



%% Integrate one more cycle to determine amplitude
tauEnd = tauInitial;
uList_ss = []; tauList_ss = [];
while tauInitial < tauEnd + T

    if state == 1 % closed state
        
        du1 = @(tau1,u1) [u1(2);rho^2*sin(rho*tau1) - 2*zeta*u1(2) - u1(1)];
        [tau1,u1,taue,ue,ie] = ode45(du1,[tauInitial,tauEnd + T],[IC],options1);
        
        tauList_ss = [tauList_ss;tau1];
        uList_ss = [uList_ss;u1(:,1)];
        
        tauInitial = tau1(end); % Initial time for next stage
        IC = u1(end,:); % Update initial conditions
        
        % Determine next state
        if ie == 1
            state = 2;
        else
%             disp('No transition')
        end
        %--------------------------------------------------------------------------------
    elseif state == 2 % open state
        
        du2 = @(tau2,u2) [u2(2);rho^2*sin(rho*tau2) - (2*zeta+2*zeta_star*rho_star)*u2(2) - (1+rho_star^2)*u2(1) + delta*rho_star^2];
        [tau2,u2,taue,ue,ie] = ode45(du2,[tauInitial,tauEnd + T],[IC],options2);
        
        tauList_ss = [tauList_ss;tau2];
        uList_ss = [uList_ss;u2(:,1)];
        
        tauInitial = tau2(end); % Initial time for next stage
        IC = u2(end,:); % Update initial conditions
        
        % Update the state
        if ie == 1
            state = 1;
        else
%             disp('No transition')
        end
        
        %--------------------------------------------------------------------------------
    else
        disp('Unexpected state determination')
    end

   
end
Amp = (max(uList_ss) - min(uList_ss))/2;

%%

end

