clear all
close all
clc
%% addpath for functions
addpath('functions');
addpath('Initial guesses');
load('InitialGuesses.mat');
%%
global zeta rho theta_o zeta_bar rho_bar theta_c Nsub T delta delta_bar
%% Options
Nsub = 1; %periodic-N motion
NumOfResult = 1;
%% Parameters
delta = 0;

zeta = 0.015;
zeta_star = 0.015;
rho_star = 2;

zeta_bar = (zeta + zeta_star*rho_star)/sqrt(1 + rho_star^2);
rho_bar = sqrt(1 + rho_star^2);
delta_bar = delta*(rho_star^2/(1+rho_star^2));

%% Freq. range
rho_l = 1.0;
rho_h = 2.4;
n = 1000;

rhoList = linspace(rho_l,rho_h,n);
% rhoList = fliplr(rhoList );

%% Options
ResnCutoff = 1e-8
options = optimset('FinDiffType','central','TolFun',1.0e-8,'TolX',1.0e-8,'Display','off'); %Nonlinear solver
fau = 1; %upper limit of Ts
fal = 0; %lower limit of Ts

U0 = InitialGuesses(NumOfResult,3:end);

%% Set up for the boundary for th nonlinear solver
lb = [-inf, -inf, -inf, -inf, -inf, 0];
ub = [ inf,  inf,  inf,  inf, inf, 0];

%% Preestablish some matrice
UampList = []; Utran_List = []; Xss_List = []; Phase_List = [];

%% Sweep process
for i = 1:n
    i
    rho = rhoList(i);
    T = 2*pi/rho;
    theta_o = atan2(2*zeta*rho,1-rho^2);
    theta_c = atan2(2*zeta_bar*rho*rho_bar,rho_bar^2-rho^2);
    
    %% Nonlinear solver
    ub(end) = T*fau; lb(end) = T*fal;
    [U,resnorm] = lsqnonlin(@sol_lsqnonlin,U0,lb,ub,options);
    resn = norm(resnorm); %if need
    U0 = U;
    
    %% Construct a vibration cycle
    Tc = U(end);
    tcList = linspace(0,Tc,1000);
    toList = linspace(Tc,Nsub*T,1000);
    tList = [tcList,toList];
    
    %Transient Response
    uc_trans = exp(-zeta_bar*rho_bar.*tcList).*U(3).*sin(sqrt(1-zeta_bar^2)*rho_bar.*tcList + U(4));
    uo_trans = exp(-zeta.*toList)*U(1).*sin(sqrt(1-zeta^2).*toList + U(2));
    u_trans = [uc_trans,uo_trans];%Transient Response
    
    %Steady State Response
    uc_stead = ((rho/rho_bar)^2)*sin(rho.*tcList-theta_c+U(5))/sqrt((1-(rho/rho_bar)^2)^2+(2*zeta_bar*rho/rho_bar)^2);
    uo_stead = rho^2*sin(rho.*toList-theta_o+U(5))/sqrt((1-rho^2)^2+(2*zeta*rho)^2);
    x_stead = [uc_stead,uo_stead];%Steady State Response
    
    %Total Response
    uc_total = uc_trans+uc_stead + delta_bar;
    uo_total = uo_trans+uo_stead;
    u_total = [uc_total,uo_total];%:= x_total = x_trans+x_stead;
      
    % Amplitude
    Uamp = (max(u_total)-min(u_total))/2;
    % Transient amplitude
    Uc_tran = abs(U(3));
    Uo_tran = abs(U(1));

    %% Break if residual is too large
    if resn>ResnCutoff
        break
    end
   %% Save results 
   UampList = [UampList;[rho,Uamp,resn,Tc/T]];
   Utran_List = [Utran_List;Uc_tran,Uo_tran]; 
   Phase_List = [Phase_List;U(5)];
   
    
end

%% Linear responses
G1 = rhoList.^2./sqrt((1-rhoList.^2).^2+(2*zeta*rhoList).^2);
G2 = abs(rhoList.^2./(-rhoList.^2 + j*rhoList*2*(zeta + zeta_star*rho_star) + (1 + rho_star^2)));
LinearResp_o = [rhoList;G1];
LinearResp_c = [rhoList;G2];

%% Plot 
figure(1) %Freq v.s. Amp
hold on
plot(UampList(:,1),UampList(:,2),'b','linewidth',1.5)
plot(rhoList,G1,'r:','linewidth',1.5)
plot(rhoList,G2,'k:','linewidth',1.5)

box on
xlabel('Excitation frequency (\rho)')
ylabel('Amplitude (u_A_M_P)')
title(['Force response (delta = ',num2str(delta),')'])

