clear all
close all
clc
%% addpath for functions
addpath('functions');
%%
global zeta rho theta_o zeta_bar rho_bar theta_c Nsub T delta delta_bar
%% Options
Nsub = 1; % periodic-N motion
NumOfResult = 1; % the number of the result to be plotted
%% Parameters
rho = 1.0;
delta = 0;

zeta = 0.015;
zeta_star = 0.015;
rho_star = 2;

N = 10; % Number of initial guesses

zeta_bar = (zeta + zeta_star*rho_star)/sqrt(1 + rho_star^2);
rho_bar = sqrt(1 + rho_star^2);
delta_bar = delta*(rho_star^2/(1+rho_star^2));
T = 2*pi/rho;

%% Options
options = optimset('FinDiffType','central','TolFun',1.0e-8,'TolX',1.0e-10,'Display','off'); %Nonlinear solver
fau = 0.99; %upper limit of Ts
fal = 0.01; %lower limit of Ts

%% Set up for the boundary for th nonlinear solver
lb = [-inf, -inf, -inf, -inf, -2*pi,Nsub*T*fal];
ub = [ inf,  inf,  inf,  inf, 2*pi,Nsub*T*fau];

%% Preestablish some matrice
UList = [];
faList = [];
ResnList = [];

%% Damping, frequency, some pgase angles
theta_o=atan2(2*zeta*rho,1-rho^2);
theta_c=atan2(2*zeta_bar*rho*rho_bar,rho_bar^2-rho^2);

%% Nonlinear solver
for i = 1:N
    U0 = [2*(rand(1,4)-0.5), pi/0.5*(rand-0.5), rand*(Nsub*T)]; % Use random number generator in foor loop
    [U,resnorm] = lsqnonlin(@sol_lsqnonlin,U0,lb,ub,options);
    Ts = U(end);
    fa = Ts/(Nsub*T);
    resn = norm(resnorm); 
    UList = [UList;U];
    faList = [faList;fa];
    ResnList = [ResnList;resn];
end
InitialGuesses = [ResnList,faList,UList];
InitialGuesses = sortrows(InitialGuesses,1); % sort rows of the matrix in ascending order based on the first column

%% Plot a vibration cycle
U = InitialGuesses(NumOfResult,3:end);

Tc = U(end);
tcList = linspace(0,Tc,100);
toList = linspace(Tc,Nsub*T,100);
tList = [tcList,toList];

%Transient Response
uc_trans = exp(-zeta_bar*rho_bar.*tcList).*U(3).*sin(sqrt(1-zeta_bar^2)*rho_bar.*tcList + U(4));
uo_trans = exp(-zeta.*toList)*U(1).*sin(sqrt(1-zeta^2).*toList + U(2));
u_trans = [uc_trans,uo_trans];%Transient Response

%Steady State Response
uc_stead = ((rho/rho_bar)^2)*sin(rho.*tcList-theta_c+U(5))/sqrt((1-(rho/rho_bar)^2)^2+(2*zeta_bar*rho/rho_bar)^2);
uo_stead = rho^2*sin(rho.*toList-theta_o+U(5))/sqrt((1-rho^2)^2+(2*zeta*rho)^2);
x_stead = [uc_stead,uo_stead];%Steady State Response

%Total Response
uc_total = uc_trans+uc_stead + delta_bar;
uo_total = uo_trans+uo_stead;
u_total = [uc_total,uo_total];%:= x_total = x_trans+x_stead;

%Velocity
uc_prime = -zeta_bar*rho_bar.*exp(-zeta_bar*rho_bar.*tcList)*U(3).*sin(sqrt(1-zeta_bar^2)*rho_bar.*tcList + U(4))...
    + sqrt(1-zeta_bar^2)*rho_bar*U(3).*exp(-zeta_bar*rho_bar.*tcList).*cos(sqrt(1-zeta_bar^2)*rho_bar.*tcList + U(4))...
    + rho*(rho/rho_bar)^2*cos(rho*tcList-theta_c+U(5))/sqrt((1-(rho/rho_bar)^2)^2+(2*zeta_bar*rho/rho_bar)^2);
uo_prime = -zeta*U(1)*exp(-zeta.*toList).*sin(sqrt(1-zeta_bar^2).*toList + U(2))...
    + sqrt(1-zeta^2)*U(1)*exp(-zeta.*toList).*cos(sqrt(1-zeta^2).*toList+U(2))...
    + rho^3.*cos(rho*toList-theta_o+U(5))/sqrt((1-rho^2)^2+(2*zeta*rho)^2);
u_prime_total = [uc_prime,uo_prime];

%Plot
figure(1) 
hold on
plot(tList,u_total,'-','linewidth',1.5,'color','b')

X=xlabel('Time [s]');
Y=ylabel('Displacement [m]');
title('Displacement response')
box on

%% Save file
InitialGuesses = InitialGuesses(1,:); % Only save the first set of solution
save('.\Initial guesses\InitialGuesses.mat','InitialGuesses');

disp(['Residual = ' num2str(InitialGuesses(NumOfResult,1))])
disp(['Time fraction = ' num2str(InitialGuesses(NumOfResult,2))])
disp(['Initial velocity = ' num2str(u_prime_total(1)) ' (Only physically valid when it is positive since the first state is assumed to be closed)'])


