clear all
close all
clc
%% addpath for functions
addpath('functions');
addpath('Initial guesses');
load('InitialGuesses.mat');
%%
global zeta rho theta_o zeta_bar rho_bar theta_c Nsub T delta delta_bar

%% Parameters
delta = 0; % Frequency sweep for delta = 0
zeta = 0.015;
zeta_star = 0.015;
% rho_star = sqrt(2);
rho_star = 2;

zeta_bar = (zeta + zeta_star*rho_star)/sqrt(1 + rho_star^2);
rho_bar = sqrt(1 + rho_star^2);
delta_bar = delta*(rho_star^2/(1+rho_star^2));

rho_bi = 2*1*sqrt(1 + rho_star^2)/(1 + sqrt(1 + rho_star^2)); % bilinear frequency

%% Frequency sweep range
rho_l = 1.0;
rho_h = 2.24;
N_freq = 300;

rhoList = linspace(rho_l,rho_h,N_freq);
% rhoList = fliplr(rhoList );

%% Gap size sweep options
delta_l = -250;
delta_h = 50;
N_gap = 1000;

%% Options for BAA computational process
ResnCutoff = 1e-8
options = optimset('FinDiffType','central','TolFun',1.0e-8,'TolX',1.0e-10,'Display','off'); %Nonlinear solver
fau = 1; %upper limit of Tc
fal = 0; %lower limit of Tc
Nsub = 1; %periodic-N motion
NumOfIG = 1; 
U0 = InitialGuesses(NumOfIG,3:end);

% Set up for the boundary for the nonlinear solver
lb = [-inf, -inf, -inf, -inf, -inf, 0]; % The last value will be replaced in the for loop
ub = [ inf,  inf,  inf,  inf, inf, 0];

%% Preestablish some matrice
UampList = []; Utran_List = []; Xss_List = []; Phase_List = [];  FreqSweepResult  = [];

%% (1) Frequency sweep for gap (delta) = 0
for i = 1:N_freq
    disp(['Frequency sweep process (frequency point: ' num2str(i) ')'])
    rho = rhoList(i);
    T = 2*pi/rho;
    theta_o = atan2(2*zeta*rho,1-rho^2);
    theta_c = atan2(2*zeta_bar*rho*rho_bar,rho_bar^2-rho^2);
    
    %% Nonlinear solver
    ub(end) = T*fau; lb(end) = T*fal;
    [U,resnorm] = lsqnonlin(@sol_lsqnonlin,U0,lb,ub,options);
    resn = norm(resnorm); %if need
    %% Break if residual is too large
    if resn>ResnCutoff
        break
    end
    U0 = U; % Update initial guesses for next frequency point
    %% Save parameter values
    FreqSweepResult = [FreqSweepResult;[rho,U]]; % Save for gap sweep process
    
    %% Construct a vibration cycle for plot
    Tc = U(end);
    tcList = linspace(0,Tc,1000);
    toList = linspace(Tc,Nsub*T,1000);
    tList = [tcList,toList];
    
    %Transient Response
    uc_trans = exp(-zeta_bar*rho_bar.*tcList).*U(3).*sin(sqrt(1-zeta_bar^2)*rho_bar.*tcList + U(4));
    uo_trans = exp(-zeta.*toList)*U(1).*sin(sqrt(1-zeta^2).*toList + U(2));
    u_trans = [uc_trans,uo_trans];%Transient Response
    
    %Steady State Response
    uc_stead = ((rho/rho_bar)^2)*sin(rho.*tcList-theta_c+U(5))/sqrt((1-(rho/rho_bar)^2)^2+(2*zeta_bar*rho/rho_bar)^2);
    uo_stead = rho^2*sin(rho.*toList-theta_o+U(5))/sqrt((1-rho^2)^2+(2*zeta*rho)^2);
    x_stead = [uc_stead,uo_stead];%Steady State Response
    
    %Total Response
    uc_total = uc_trans+uc_stead + delta_bar;
    uo_total = uo_trans+uo_stead;
    u_total = [uc_total,uo_total];%:= x_total = x_trans+x_stead;
    
   
    % Amplitude
    Uamp = (max(u_total)-min(u_total))/2;
    % Transient amplitude
    Uc_tran = abs(U(3));
    Uo_tran = abs(U(1));

    
   %% Save results 
   UampList = [UampList;[rho,Uamp,resn,Tc/T]];
   Utran_List = [Utran_List;Uc_tran,Uo_tran]; 
   Phase_List = [Phase_List;U(5)];
   
    
end

%% Linear responses
G1 = rhoList.^2./sqrt((1-rhoList.^2).^2+(2*zeta*rhoList).^2);
G2 = abs(rhoList.^2./(-rhoList.^2 + j*rhoList*2*(zeta + zeta_star*rho_star) + (1 + rho_star^2)));

%% Plot 
figure(1) %Freq v.s. Amp
hold on
plot(UampList(:,1),UampList(:,2),'b','linewidth',1.5)
plot(rhoList,G1,'r:','linewidth',1.5)
plot(rhoList,G2,'k:','linewidth',1.5)

box on
xlabel('Excitation frequency (\rho)')
ylabel('Amplitude (u_A_M_P)')
title(['Frequency sweep (\delta = ',num2str(delta),') & Maximum response envolope'])
axis([rho_l,rho_h,0,max(G1)*1.2])


%% (2) Sweep through different gap size to determine the maximum response and corresponding gap size
GapSweepResult = [];
for i = 1:N_freq
    disp(['Optimized gap size calculations (frequency point: ' num2str(i) ')'])
    rho = rhoList(i);
    T = 2*pi/rho;
    theta_o = atan2(2*zeta*rho,1-rho^2);
    theta_c = atan2(2*zeta_bar*rho*rho_bar,rho_bar^2-rho^2);
    U0 = FreqSweepResult(i,2:end);
    
    % Constrcut deltaList for cases where system has gap & prestress
    if rho < rho_bi
        deltaList = linspace(0,delta_h,N_gap);
    elseif rho > rho_bi
        deltaList = linspace(0,delta_l,N_gap);
    end
    UampList = [];  Utran_List = [];
    
    for j = 1:N_gap
        delta = deltaList(j); %gap
        delta_bar = delta*(rho_star^2/(1+rho_star^2));
        
        %% Nonlinear solver
        ub(end) = T*fau; lb(end) = T*fal;
        [U,resnorm] = lsqnonlin(@sol_lsqnonlin,U0,lb,ub,options);
        resn = norm(resnorm); %if need
                
        %% Break if residual is too large
        if resn>ResnCutoff
            % Save the maximum amplitude and corresponding gap size
            [M,I] = max(UampList(:,2));
            GapSweepResult = [GapSweepResult;[rho,UampList(I,1),M,UampList(I,4)]]; % [rho, gap size (delta), maximum amplitude, time fraction]
            break
        end
        
        U0 = U; % Update initial guesses
        %% Construct a vibration cycle
        Tc = U(end);
        tcList = linspace(0,Tc,1000);
        toList = linspace(Tc,Nsub*T,1000);
        tList = [tcList,toList];
        
        %Transient Response
        uc_trans = exp(-zeta_bar*rho_bar.*tcList).*U(3).*sin(sqrt(1-zeta_bar^2)*rho_bar.*tcList + U(4));
        uo_trans = exp(-zeta.*toList)*U(1).*sin(sqrt(1-zeta^2).*toList + U(2));
        u_trans = [uc_trans,uo_trans];%Transient Response
        
        %Steady State Response
        uc_stead = ((rho/rho_bar)^2)*sin(rho.*tcList-theta_c+U(5))/sqrt((1-(rho/rho_bar)^2)^2+(2*zeta_bar*rho/rho_bar)^2);
        uo_stead = rho^2*sin(rho.*toList-theta_o+U(5))/sqrt((1-rho^2)^2+(2*zeta*rho)^2);
        x_stead = [uc_stead,uo_stead];%Steady State Response
        
        %Total Response
        uc_total = uc_trans+uc_stead + delta_bar;
        uo_total = uo_trans+uo_stead;
        u_total = [uc_total,uo_total];%:= x_total = x_trans+x_stead;
        
        %Velocity
        uc_prime = -zeta_bar*rho_bar.*exp(-zeta_bar*rho_bar.*tcList)*U(3).*sin(sqrt(1-zeta_bar^2)*rho_bar.*tcList + U(4))...
            + sqrt(1-zeta_bar^2)*rho_bar*U(3).*exp(-zeta_bar*rho_bar.*tcList).*cos(sqrt(1-zeta_bar^2)*rho_bar.*tcList + U(4))...
            + rho*(rho/rho_bar)^2*cos(rho*tcList-theta_c+U(5))/sqrt((1-(rho/rho_bar)^2)^2+(2*zeta_bar*rho/rho_bar)^2);
        uo_prime = -zeta*U(1)*exp(-zeta.*toList).*sin(sqrt(1-zeta_bar^2).*toList + U(2))...
            + sqrt(1-zeta^2)*U(1)*exp(-zeta.*toList).*cos(sqrt(1-zeta^2).*toList+U(2))...
            + rho^3.*cos(rho*toList-theta_o+U(5))/sqrt((1-rho^2)^2+(2*zeta*rho)^2);
        u_prime_total = [uc_prime,uo_prime];
        
        % Amplitude
        Uamp = (max(u_total)-min(u_total))/2;
        % Transient amplitude
        Uc_tran = abs(U(3));
        Uo_tran = abs(U(1));

        %% Save results
        UampList = [UampList;[delta,Uamp,resn,Tc/T]];
        Utran_List = [Utran_List;Uc_tran,Uo_tran];
        
        
    end
end
% Plot amplitude envolope on first figure
plot(GapSweepResult(:,1),GapSweepResult(:,3),'g-','linewidth',1.5)

% plot corresponding gap size
figure(2)
plot(GapSweepResult(:,1),GapSweepResult(:,2),'g','linewidth',1.5)
xlabel('Excitation frequency (\rho)')
ylabel('Gap size that maxizes response')

